/*************************************************************************
 *  Dateiname    ; main.c
 *
 *  Beschreibung : Testumgebung fr xeeprom.h auf einem XPLAIN Board
 *
 *  Autor        : Joachim Rath
 *  Revision     : initial 04/2010
 *
 *************************************************************************
 *                 (C) Copyright 2010 Joachim Rath
 *                Joachim[dot]Rath[at]JoRath[dot]eu
 *************************************************************************
 * Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation
 * verffentlicht, weitergeben und/oder modifizieren, entweder gem
 * Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version.
 *
 * Die Verffentlichung dieses Programms erfolgt in der Hoffnung,
 * da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
 * sogar ohne die implizite Garantie der MARKTREIFE oder der
 * VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der
 * GNU General Public License.
 *
 * Sie sollten eine Kopie der GNU General Public License zusammen
 * mit diesem Programm erhalten haben.
 * Falls nicht, schreiben Sie an die Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *************************************************************************/
#include <avr/io.h>             //AVR device-specific IO definitions
#include <string.h>             //Strings
//#include <avr/eeprom.h>         //EEPROM handling

#include "xeeprom.h"

/*************************************************************************
 *                           Portzuweisungen
 *************************************************************************/
#define LED0_ON             (PORTE_OUTCLR=PIN0_bm)
#define LED1_ON             (PORTE_OUTCLR=PIN1_bm)
#define LED2_ON             (PORTE_OUTCLR=PIN2_bm)
#define LED3_ON             (PORTE_OUTCLR=PIN3_bm)
#define LED4_ON             (PORTE_OUTCLR=PIN4_bm)
#define LED5_ON             (PORTE_OUTCLR=PIN5_bm)
#define LED6_ON             (PORTE_OUTCLR=PIN6_bm)
#define LED7_ON             (PORTE_OUTCLR=PIN7_bm)

#define LED0_OFF            (PORTE_OUTSET=PIN0_bm)
#define LED1_OFF            (PORTE_OUTSET=PIN1_bm)
#define LED2_OFF            (PORTE_OUTSET=PIN2_bm)
#define LED3_OFF            (PORTE_OUTSET=PIN3_bm)
#define LED4_OFF            (PORTE_OUTSET=PIN4_bm)
#define LED5_OFF            (PORTE_OUTSET=PIN5_bm)
#define LED6_OFF            (PORTE_OUTSET=PIN6_bm)
#define LED7_OFF            (PORTE_OUTSET=PIN7_bm)

#define LED0_TGL            (PORTE_OUTTGL=PIN0_bm)
#define LED1_TGL            (PORTE_OUTTGL=PIN1_bm)
#define LED2_TGL            (PORTE_OUTTGL=PIN2_bm)
#define LED3_TGL            (PORTE_OUTTGL=PIN3_bm)
#define LED4_TGL            (PORTE_OUTTGL=PIN4_bm)
#define LED5_TGL            (PORTE_OUTTGL=PIN5_bm)
#define LED6_TGL            (PORTE_OUTTGL=PIN6_bm)
#define LED7_TGL            (PORTE_OUTTGL=PIN7_bm)

/*************************************************************************
 *                            Definitionen
 *************************************************************************/
/*************************************************************************
 *                             Prototypen
 *************************************************************************/
/*************************************************************************
 *                          globale Variablen
 *************************************************************************/
/*************************************************************************
 *                           lokale Variablen
 *************************************************************************/
/*************************************************************************
 *
 *  Name         : InitIO
 *
 *  Beschreibung : LEDs und Taster einrichten
 *                 Wird direkt ber dem Reset-Vektor aufgerufen, d.h.
 *                 der Stack-Pointer ist noch nicht initialisiert, also
 *                 nur direkte Zuweisungen benutzen. GCC geht davon aus
 *                 das R1 immer 0 ist!
 *
 *  Parameter    : keine
 *
 *  Rckgabe     : keine
 *
 *************************************************************************/
void __attribute__ ((naked, section(".init0"))) XplainIO (void)
{
 __asm ("eor r1, r1");                  // Register 1 lschen

 PORTE.DIR      = 0xff;                 // LEDs auf Ausgang
 PORTE.OUT      = 0xff;                 // LEDs aus
 PORTF.DIR      = 0x00;                 // Taster auf Eingang
 PORTF.OUT      = 0xff;                 // PullUps ein
 LED0_ON;                               // Programm ist gestartet...
}
/*************************************************************************
 *
 *  Name         : .init.3
 *
 *  Parameter    : keine
 *
 *  Rckgabe     : keine
 *
 *************************************************************************/
void __attribute__ ((naked, section(".init3"))) XplainMemAndClk(void)
{

 LED1_ON;                               // Konfiguration beginnt...

 EEpromInit();

 LED2_ON;                                // ... geschafft
}
/*************************************************************************
 *
 *  Name         : main
 *
 *  Beschreibung : nicht ntig!
 *
 *  Parameter    : keine
 *
 *
 *  Rckgabe     : 0
 *
 *************************************************************************/
int __attribute__((OS_main)) main(void)
{
  LED3_ON;

  static uint8_t  nByte  EEMEM;
  static uint16_t nWord  EEMEM;
  static uint32_t nDWord EEMEM;
  static float    nFloat EEMEM;
  static uint8_t  nBlock[10] EEMEM;
  static uint8_t  oByte  EEMEM;
  static uint16_t oWord  EEMEM;
  static uint32_t oDWord EEMEM;
  static float    oFloat EEMEM;
  static uint8_t  oBlock[10] EEMEM;

  static uint8_t  rByte;
  static uint16_t rWord;
  static uint32_t rDWord;
  static float    rFloat;
  static uint8_t  rBlock[10];

  EEpromWriteByte(&nByte,0xab);
  rByte = EEpromReadByte(&nByte);

  if(rByte!=0xab)
   LED3_OFF;

  EEpromWriteWord(&nWord,0xabcd);
  rWord = EEpromReadWord(&nWord);

  if(rWord!=0xabcd)
   LED3_OFF;

  EEpromWriteDWord(&nDWord,0xdeadbeaf);
  rDWord = EEpromReadDWord(&nDWord);

  if(rDWord!=0xdeadbeaf)
   LED3_OFF;

  EEpromWriteFloat(&nFloat,1.2345);
  rFloat = EEpromReadFloat(&nFloat);

  if(rFloat!=1.2345)
   LED3_OFF;

  EEpromWriteBlock("1234567890",&nBlock,10);
  EEpromReadBlock(&rBlock,&nBlock,10);

  if(memcmp("1234567890",&rBlock,10))
   LED3_OFF;

  eeprom_write_byte(&oByte,0xab);
  rByte = eeprom_read_byte(&oByte);

  if(rByte!=0xab)
   LED3_OFF;

  eeprom_write_word(&oWord,0xabcd);
  rWord = eeprom_read_word(&oWord);

  if(rWord!=0xabcd)
   LED3_OFF;

  eeprom_write_dword(&oDWord,0xdeadbeaf);
  rDWord = eeprom_read_dword(&oDWord);

  if(rDWord!=0xdeadbeaf)
   LED3_OFF;

  eeprom_write_float(&oFloat,1.2345);
  rFloat = eeprom_read_float(&oFloat);

  if(rFloat!=1.2345)
   LED3_OFF;

  eeprom_write_block("1234567890",&oBlock,10);
  eeprom_read_block(&rBlock,&oBlock,10);

  if(memcmp("1234567890",&rBlock,10))
   LED3_OFF;
  



/*************************************************************************
 *  main Loop
 *************************************************************************/
 while(1)
 {}
}
/*************************************************************************
 *                      E N D   O F   S O U R C E
 *************************************************************************/

